@@ -2,7 +2,7 @@ import logging
 import re
 import sys
 import os
-from dataclasses import fields
+from dataclasses import fields, is_dataclass
 from pathlib import Path
 import importlib.metadata
 from typing import Optional
@@ -10,7 +10,7 @@ from typing import Optional
 import typer
 import git
 from git import Repo
-
+from .env import Env
 
 _EXT_TO_HINT: dict[str, str] = {
     # scripting & languages
@@ -194,10 +194,10 @@ def detect_github_env() -> dict:
         "github_ref": ref,
     }
     # Fallback for local usage: try to get from git
-    if not repo:
+    if not repo or repo == "octocat/Hello-World":
         git_repo = None
         try:
-            git_repo = Repo(".", search_parent_directories=True)
+            git_repo = Repo(Env.working_folder, search_parent_directories=True)
             origin = git_repo.remotes.origin.url
             # e.g. git@github.com:Nayjest/ai-code-review.git -> Nayjest/ai-code-review
             match = re.search(r"[:/]([\w\-]+)/([\w\-\.]+?)(\.git)?$", origin)
@@ -254,6 +254,9 @@ def filter_kwargs(cls, kwargs, log_warnings=True):
     Returns:
         A dictionary containing only the fields that are defined in the dataclass.
     """
+    if not is_dataclass(cls):
+        raise TypeError(f"{cls.__name__} is not a dataclass or pydantic dataclass")
+
     cls_fields = {f.name for f in fields(cls)}
     filtered = {}
     for k, v in kwargs.items():
